@@ -1,5 +1,5 @@
+use asdi::edb::io::{print_relation, Format};
 use asdi::idb::eval::{Evaluator, NaiveEvaluator};
-use asdi::io::{print_relation, Format};
 use asdi::parse::parse_file;
 use asdi::{Collection, Labeled};
 
