@@ -1,6 +1,8 @@
 /*!
 This module provides the set of types that primarily describe the Extensional Database (EDB).
 
+![module UML](https://raw.githubusercontent.com/johnstonskj/rust-asdi/main/book/src/model/edb.svg)
+
 Given the following fact:
 
 ```datalog
@@ -93,28 +95,30 @@ boolean_fact(false).
 */
 
 use crate::error::{
-    fact_does_not_correspond_to_schema, invalid_value, nullary_facts_not_allowed, relation_exists,
-    Error, Result,
+    attribute_does_not_exist, attribute_index_invalid, fact_does_not_correspond_to_schema,
+    invalid_value, nullary_facts_not_allowed, relation_exists, Error, Result,
 };
 use crate::idb::query::relational::{FactOps, Projection, Selection};
 use crate::idb::query::{Queryable, Row, View};
 use crate::idb::Atom;
-use crate::io::{read_relation, write_relation, FilePragma};
 use crate::syntax::{
-    ANONYMOUS_COLUMN_NAME, BOOLEAN_LITERAL_FALSE, BOOLEAN_LITERAL_TRUE, CHAR_COLON, CHAR_COMMA,
-    CHAR_LEFT_PAREN, CHAR_PERIOD, CHAR_RIGHT_PAREN, CHAR_UNDERSCORE, PRAGMA_ID_ASSERT,
-    PRAGMA_ID_INFER, RESERVED_PREFIX, TYPE_NAME_PREDICATE,
+    ANONYMOUS_COLUMN_NAME, BOOLEAN_LITERAL_FALSE, BOOLEAN_LITERAL_TRUE, CHAR_COLON,
+    CHAR_LEFT_PAREN, CHAR_PERIOD, CHAR_RIGHT_PAREN, CHAR_UNDERSCORE, COMMA_SEPARATOR,
+    FUNCTIONAL_DEPENDENCY_UNICODE_SYMBOL, PRAGMA_ID_ASSERT, PRAGMA_ID_INFER, RESERVED_PREFIX,
+    TYPE_NAME_PREDICATE,
 };
 use crate::{
-    AttributeName, AttributeNameRef, Collection, IndexedCollection, Labeled, MaybeAnonymous,
-    MaybeLabeled, Term, Variable,
+    AttributeName, AttributeNameRef, Collection, IndexedCollection, Labeled, MaybeAnonymous, Term,
+    Variable,
 };
+use ordered_float::NotNan;
 use paste::paste;
 use std::collections::{BTreeMap, HashSet};
 use std::fmt::{Debug, Display, Formatter};
+use std::ops::Sub;
 use std::rc::Rc;
 use std::str::FromStr;
-use tracing::trace;
+use tracing::{error, trace};
 
 // ------------------------------------------------------------------------------------------------
 // Public Types & Constants
@@ -135,7 +139,7 @@ pub struct Relation {
     label: PredicateRef,
     schema: Schema<Predicate>,
     facts: HashSet<Fact>,
-    pragma: Option<FilePragma>,
+    pragma: Option<io::FilePragma>,
 }
 
 // ------------------------------------------------------------------------------------------------
@@ -143,13 +147,20 @@ pub struct Relation {
 ///
 /// A schema is an ordered list of [attribute descriptions](Attribute).
 ///
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Schema<T>
 where
     T: AttributeName,
 {
     attributes: Vec<Attribute<T>>,
     label_index: BTreeMap<AttributeNameRef<T>, usize>,
+    functional_dependencies: HashSet<FunctionalDependency>,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub struct FunctionalDependency {
+    determinant: Vec<usize>,
+    dependent: Vec<usize>,
 }
 
 ///
@@ -186,6 +197,7 @@ where
 pub enum AttributeKind {
     String,
     Integer,
+    Float,
     Boolean,
 }
 
@@ -208,10 +220,13 @@ pub struct Fact {
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub enum Constant {
     String(String),
-    Integer(i64),
+    Number(Number),
     Boolean(bool),
 }
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub struct Number(NumberInner);
+
 // ------------------------------------------------------------------------------------------------
 
 ///
@@ -238,6 +253,12 @@ pub type PredicateRef = Rc<Predicate>;
 // Private Types & Constants
 // ------------------------------------------------------------------------------------------------
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub enum NumberInner {
+    Integer(i64),
+    Float(NotNan<f64>),
+}
+
 // ------------------------------------------------------------------------------------------------
 // Private Macros
 // ------------------------------------------------------------------------------------------------
@@ -341,7 +362,6 @@ where
             .collect();
         let pre_hash_len = named.len();
 
-        // BUG: This removes duplicate unnamed variables!!!
         let named = BTreeMap::from_iter(named);
 
         assert_eq!(named.len(), pre_hash_len);
@@ -349,6 +369,7 @@ where
         Self {
             attributes,
             label_index: named,
+            functional_dependencies: Default::default(),
         }
     }
 
@@ -356,6 +377,7 @@ where
         Self {
             attributes: Default::default(),
             label_index: Default::default(),
+            functional_dependencies: Default::default(),
         }
     }
 
@@ -415,7 +437,109 @@ where
             .iter()
             .map(|a| a.to_column_decl(emit_unknown))
             .collect::<Vec<String>>()
-            .join(&format!("{} ", CHAR_COMMA))
+            .join(COMMA_SEPARATOR)
+    }
+
+    // --------------------------------------------------------------------------------------------
+
+    pub fn attribute_index_to_index(&self, index: &AttributeIndex<T>) -> Result<usize> {
+        match index {
+            AttributeIndex::Label(label) => {
+                if let Some(index) = self.label_to_index(label) {
+                    Ok(index)
+                } else {
+                    Err(attribute_does_not_exist(label.to_string()))
+                }
+            }
+            AttributeIndex::Index(index) => {
+                if *index < self.attributes.len() {
+                    Ok(*index)
+                } else {
+                    Err(attribute_index_invalid(*index))
+                }
+            }
+        }
+    }
+
+    pub fn has_functional_dependencies(&self) -> bool {
+        !self.functional_dependencies.is_empty()
+    }
+
+    pub fn functional_dependencies(&self) -> &HashSet<FunctionalDependency> {
+        &self.functional_dependencies
+    }
+
+    pub fn add_functional_attribute(&mut self, determinant: AttributeIndex<T>) -> Result<bool> {
+        let determinant = self.attribute_index_to_index(&determinant)?;
+        let dependents: Vec<usize> = (0..self.attributes.len().sub(determinant)).collect();
+
+        Ok(self
+            .functional_dependencies
+            .insert(FunctionalDependency::new(vec![determinant], dependents)))
+    }
+
+    pub fn add_functional_dependency<V: Into<Vec<AttributeIndex<T>>>>(
+        &mut self,
+        determinant: V,
+        dependent: V,
+    ) -> Result<bool> {
+        let determinant = self.attribute_indices_to_index(&determinant.into())?;
+        let determinant_hash: HashSet<usize> = HashSet::from_iter(determinant.iter().copied());
+        assert_eq!(determinant.len(), determinant_hash.len());
+
+        let dependent = self.attribute_indices_to_index(&dependent.into())?;
+        let dependent_hash: HashSet<usize> = HashSet::from_iter(dependent.iter().cloned());
+        assert_eq!(dependent.len(), dependent_hash.len());
+
+        assert_eq!(determinant_hash.intersection(&dependent_hash).count(), 0);
+
+        Ok(self
+            .functional_dependencies
+            .insert(FunctionalDependency::new(determinant, dependent)))
+    }
+
+    fn attribute_indices_to_index(&self, indices: &[AttributeIndex<T>]) -> Result<Vec<usize>> {
+        indices
+            .iter()
+            .map(|i| self.attribute_index_to_index(i))
+            .collect()
+    }
+}
+
+// ------------------------------------------------------------------------------------------------
+
+impl Display for FunctionalDependency {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{} {} {}",
+            self.determinants()
+                .map(|d| (d + 1).to_string())
+                .collect::<Vec<String>>()
+                .join(COMMA_SEPARATOR),
+            FUNCTIONAL_DEPENDENCY_UNICODE_SYMBOL,
+            self.dependents()
+                .map(|d| (d + 1).to_string())
+                .collect::<Vec<String>>()
+                .join(COMMA_SEPARATOR),
+        )
+    }
+}
+
+impl FunctionalDependency {
+    fn new<V: Into<Vec<usize>>>(determinants: V, dependents: V) -> Self {
+        Self {
+            determinant: determinants.into(),
+            dependent: dependents.into(),
+        }
+    }
+
+    pub fn determinants(&self) -> impl Iterator<Item = &usize> {
+        self.determinant.iter()
+    }
+
+    pub fn dependents(&self) -> impl Iterator<Item = &usize> {
+        self.dependent.iter()
     }
 }
 
@@ -451,14 +575,6 @@ where
         self.label.is_none()
     }
 }
-impl<T> MaybeLabeled<T> for Attribute<T>
-where
-    T: AttributeName,
-{
-    fn label(&self) -> Option<&AttributeNameRef<T>> {
-        self.label.as_ref()
-    }
-}
 
 impl<T> Attribute<T>
 where
@@ -522,6 +638,20 @@ where
         self.index = Some(index)
     }
 
+    ///
+    /// Returns this value's label, or `None` if anonymous.
+    ///
+    pub fn label(&self) -> Option<&AttributeNameRef<T>> {
+        self.label.as_ref()
+    }
+
+    ///
+    /// Returns `true` if this value has a label, else `false`.
+    ///
+    pub fn is_labeled(&self) -> bool {
+        !self.is_anonymous()
+    }
+
     pub fn kind(&self) -> Option<AttributeKind> {
         self.kind.as_ref().copied()
     }
@@ -600,6 +730,7 @@ impl Display for AttributeKind {
             match self {
                 Self::String => "string",
                 Self::Integer => "integer",
+                Self::Float => "float",
                 Self::Boolean => "boolean",
             }
         )
@@ -613,6 +744,7 @@ impl FromStr for AttributeKind {
         match s {
             "str" | "string" => Ok(Self::String),
             "int" | "integer" => Ok(Self::Integer),
+            "float" => Ok(Self::Float),
             "bool" | "boolean" => Ok(Self::Boolean),
             _ => invalid_value("ConstantKind", s).into(),
         }
@@ -830,7 +962,7 @@ impl Queryable for Relation {
             let rows: Result<Vec<Row>> = self
                 .iter()
                 .filter_map(|fact| fact.clone().select(&selection).transpose())
-                // TODO: Need to update the view join logic
+                // TODO: (ISSUE/rust-asdi/8) Need to update the view join logic
                 // .map(|fact| match fact {
                 //     Ok(fact) => fact.project(&projection),
                 //     Err(e) => Err(e),
@@ -868,6 +1000,10 @@ impl Relation {
         &self.schema
     }
 
+    pub(crate) fn schema_mut(&mut self) -> &mut Schema<Predicate> {
+        &mut self.schema
+    }
+
     pub(crate) fn update_schema(&mut self, other: &Self) {
         assert_eq!(self.schema.len(), other.schema.len());
         self.schema
@@ -900,7 +1036,7 @@ impl Relation {
                     .iter()
                     .map(Constant::to_string)
                     .collect::<Vec<String>>()
-                    .join(&format!("{} ", CHAR_COMMA)),
+                    .join(COMMA_SEPARATOR),
             ))
         }
     }
@@ -914,7 +1050,7 @@ impl Relation {
                 fact.iter()
                     .map(Constant::to_string)
                     .collect::<Vec<String>>()
-                    .join(&format!("{} ", CHAR_COMMA)),
+                    .join(COMMA_SEPARATOR),
             ))
         }
     }
@@ -957,33 +1093,33 @@ impl Relation {
     // --------------------------------------------------------------------------------------------
 
     ///
-    /// Returns `true` if a [FilePragma] (`.input` or `.output`) is attached to this relation.
+    /// Returns `true` if a [io::FilePragma] (`.input` or `.output`) is attached to this relation.
     ///
     pub fn has_file_pragma(&self) -> bool {
         self.pragma.is_some()
     }
 
     ///
-    /// Returns a [FilePragma] if one is attached to this relation, else `None`.
+    /// Returns a [io::FilePragma] if one is attached to this relation, else `None`.
     ///
-    pub fn file_pragma(&self) -> Option<&FilePragma> {
+    pub fn file_pragma(&self) -> Option<&io::FilePragma> {
         self.pragma.as_ref()
     }
 
     ///
-    /// Attach a [FilePragma] to this relation, if one exists already it is overwritten.
+    /// Attach a [io::FilePragma] to this relation, if one exists already it is overwritten.
     ///
-    pub fn set_file_pragma(&mut self, pragma: FilePragma) {
+    pub fn set_file_pragma(&mut self, pragma: io::FilePragma) {
         self.pragma = Some(pragma)
     }
 
     ///
-    /// If a [FilePragma] is attached to this relation then read data from the file into this
+    /// If a [io::FilePragma] is attached to this relation then read data from the file into this
     /// relation's set of facts.
     ///
     pub fn load_from_file(&mut self) -> Result<usize> {
         if let Some(pragma) = &self.pragma {
-            let new_facts = read_relation(self, pragma)?;
+            let new_facts = io::read_relation(self, pragma)?;
             self.extend(new_facts)
         } else {
             Ok(0)
@@ -991,12 +1127,12 @@ impl Relation {
     }
 
     ///
-    /// If a [FilePragma] is attached to this relation then store this relation's set of facts
+    /// If a [io::FilePragma] is attached to this relation then store this relation's set of facts
     /// into the file.
     ///
     pub fn store_to_file(&mut self) -> Result<()> {
         if let Some(pragma) = &self.pragma {
-            write_relation(self, pragma)
+            io::write_relation(self, pragma)
         } else {
             Ok(())
         }
@@ -1034,7 +1170,7 @@ impl Display for Fact {
             self.iter()
                 .map(Constant::to_string)
                 .collect::<Vec<String>>()
-                .join(&format!("{} ", CHAR_COMMA)),
+                .join(COMMA_SEPARATOR),
             CHAR_RIGHT_PAREN,
             CHAR_PERIOD
         )
@@ -1116,7 +1252,21 @@ impl From<String> for Constant {
     }
 }
 
-impl_enum_from!(Constant, i64, Integer);
+impl_enum_from!(Constant, Number, Number);
+
+impl From<i64> for Constant {
+    fn from(v: i64) -> Self {
+        Self::Number(Number::from(v))
+    }
+}
+
+impl TryFrom<f64> for Constant {
+    type Error = Error;
+
+    fn try_from(v: f64) -> std::result::Result<Self, Self::Error> {
+        Ok(Self::Number(Number::try_from(v)?))
+    }
+}
 
 impl_enum_from!(Constant, bool, Boolean);
 
@@ -1132,7 +1282,7 @@ impl Display for Constant {
                     } else {
                         format!("{:?}", v)
                     },
-                Self::Integer(v) => v.to_string(),
+                Self::Number(v) => v.to_string(),
                 Self::Boolean(v) => {
                     if *v {
                         BOOLEAN_LITERAL_TRUE
@@ -1156,19 +1306,19 @@ impl Constant {
     }
 
     pub fn new_size(v: usize) -> Self {
-        Self::Integer(v as i64)
+        Self::Number(Number::from(v))
     }
 
     self_is_as!(string, String);
 
-    self_is_as!(integer, Integer, i64);
+    self_is_as!(number, Number, Number);
 
     self_is_as!(boolean, Boolean, bool);
 
     pub fn kind(&self) -> AttributeKind {
         match self {
             Self::String(_) => AttributeKind::String,
-            Self::Integer(_) => AttributeKind::Integer,
+            Self::Number(v) => v.kind(),
             Self::Boolean(_) => AttributeKind::Boolean,
         }
     }
@@ -1195,6 +1345,137 @@ impl Constant {
 
 // ------------------------------------------------------------------------------------------------
 
+impl Display for Number {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "{}",
+            match self.0 {
+                NumberInner::Integer(v) => v.to_string(),
+                NumberInner::Float(v) => v.to_string(),
+            }
+        )
+    }
+}
+
+impl From<usize> for Number {
+    fn from(v: usize) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<u8> for Number {
+    fn from(v: u8) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<i8> for Number {
+    fn from(v: i8) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<u16> for Number {
+    fn from(v: u16) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<i16> for Number {
+    fn from(v: i16) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<u32> for Number {
+    fn from(v: u32) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<i32> for Number {
+    fn from(v: i32) -> Self {
+        Self(NumberInner::Integer(v as i64))
+    }
+}
+
+impl From<i64> for Number {
+    fn from(v: i64) -> Self {
+        Self(NumberInner::Integer(v))
+    }
+}
+
+impl TryFrom<f32> for Number {
+    type Error = Error;
+
+    fn try_from(value: f32) -> std::result::Result<Self, Self::Error> {
+        Self::try_from(value as f64)
+    }
+}
+
+impl TryFrom<f64> for Number {
+    type Error = Error;
+
+    fn try_from(value: f64) -> std::result::Result<Self, Self::Error> {
+        match NotNan::new(value) {
+            Ok(float) => Ok(Self(NumberInner::Float(float))),
+            Err(e) => {
+                error!("Invalid float, cannot be a NaN: {:?}", e);
+                Err(invalid_value("float", &value.to_string()))
+            }
+        }
+    }
+}
+
+impl Number {
+    pub fn from_i64(v: i64) -> Self {
+        Self(NumberInner::Integer(v))
+    }
+
+    pub fn is_integer(&self) -> bool {
+        matches!(&self.0, NumberInner::Integer(_))
+    }
+
+    pub fn as_integer(&self) -> Option<&i64> {
+        match &self.0 {
+            NumberInner::Integer(v) => Some(v),
+            _ => None,
+        }
+    }
+
+    pub fn from_f64(v: f64) -> Option<Self> {
+        let v = NotNan::new(v);
+        match v {
+            Ok(v) => Some(Self(NumberInner::Float(v))),
+            Err(e) => {
+                error!("Could not convert to a float, error: {:?}", e);
+                None
+            }
+        }
+    }
+
+    pub fn is_float(&self) -> bool {
+        matches!(&self.0, NumberInner::Float(_))
+    }
+
+    pub fn as_float(&self) -> Option<&f64> {
+        match &self.0 {
+            NumberInner::Float(v) => Some(v.as_ref()),
+            _ => None,
+        }
+    }
+
+    pub fn kind(&self) -> AttributeKind {
+        match &self.0 {
+            NumberInner::Integer(_) => AttributeKind::Integer,
+            NumberInner::Float(_) => AttributeKind::Float,
+        }
+    }
+}
+
+// ------------------------------------------------------------------------------------------------
+
 impl Display for Predicate {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.0)
@@ -1235,3 +1516,9 @@ impl AttributeName for Predicate {
 impl Predicate {
     into_inner_fn!(String);
 }
+
+// ------------------------------------------------------------------------------------------------
+// Modules
+// ------------------------------------------------------------------------------------------------
+
+pub mod io;
