@@ -1,6 +1,8 @@
 /*!
 One-line description.
 
+![module UML](https://raw.githubusercontent.com/johnstonskj/rust-asdi/main/book/src/model/idb_eval_strata.svg)
+
 More detailed description, with
 
 # Example
@@ -10,7 +12,6 @@ More detailed description, with
 use crate::edb::{Predicate, RelationSet};
 use crate::error::{program_not_stratifiable, Result};
 use crate::features::{FeatureSet, FEATURE_DISJUNCTION};
-use crate::idb::eval::ToGraphViz;
 use crate::idb::{Literal, Rule, RuleSet};
 use crate::{Collection, Labeled, MaybePositive, Program, ProgramCore};
 use std::collections::HashSet;
@@ -53,7 +54,8 @@ pub struct StratifiedProgram<'a> {
     strata: Vec<SubProgram<'a>>,
 }
 
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
+#[allow(single_use_lifetimes)]
 pub struct SubProgram<'a> {
     program: &'a Program,
     strata: RuleSet,
@@ -120,7 +122,7 @@ impl Display for StratifiedProgram<'_> {
         for (i, strata) in self.strata.iter().enumerate() {
             writeln!(f, "[{}", i)?;
             for (j, rule) in strata.rules().iter().enumerate() {
-                writeln!(f, "  [{}] {}", j, rule)?;
+                writeln!(f, "  [{}] {:?}", j, rule)?;
             }
             writeln!(f, "]")?;
         }
@@ -128,9 +130,27 @@ impl Display for StratifiedProgram<'_> {
     }
 }
 
+impl<'a> Collection<SubProgram<'a>> for StratifiedProgram<'a> {
+    fn is_empty(&self) -> bool {
+        self.strata.is_empty()
+    }
+
+    fn len(&self) -> usize {
+        self.strata.len()
+    }
+
+    fn iter(&self) -> Box<dyn Iterator<Item = &'_ SubProgram<'a>> + '_> {
+        Box::new(self.strata.iter())
+    }
+
+    fn contains(&self, value: &SubProgram<'_>) -> bool {
+        self.strata.contains(value)
+    }
+}
+
 impl<'a> StratifiedProgram<'a> {
     pub fn from(program: &'a Program) -> Result<Self> {
-        // TODO: lift this restriction
+        // TODO: (ISSUE/rust-asdi/7) Design for disjunction
         assert!(!program.features().supports(&FEATURE_DISJUNCTION));
 
         let start = Instant::now();
@@ -207,18 +227,6 @@ impl<'a> StratifiedProgram<'a> {
             Err(program_not_stratifiable())
         }
     }
-
-    pub fn is_empty(&self) -> bool {
-        self.strata.is_empty()
-    }
-
-    pub fn len(&self) -> usize {
-        self.strata.len()
-    }
-
-    pub fn iter(&self) -> impl Iterator<Item = &SubProgram<'_>> {
-        self.strata.iter()
-    }
 }
 
 // ------------------------------------------------------------------------------------------------
@@ -252,7 +260,7 @@ impl<'a> SubProgram<'a> {
     }
 
     fn add_rule(&mut self, rule: &Rule) {
-        self.strata.add(rule.clone())
+        self.strata.add(rule.clone());
     }
 }
 
@@ -349,10 +357,8 @@ impl<'a> PrecedenceGraph<'a> {
     fn directly_reachable_nodes_from(&self, source: &Predicate) -> HashSet<&'_ PrecedenceNode<'_>> {
         self.edges().filter(|e| e.source() == source).collect()
     }
-}
 
-impl ToGraphViz for PrecedenceGraph<'_> {
-    fn to_graphviz_string(&self) -> Result<String> {
+    pub fn to_graphviz_string(&self) -> Result<String> {
         let mut edges: Vec<String> = self
             .edges()
             .map(|edge| {
@@ -444,8 +450,7 @@ fn body_predicates(rule: &Rule) -> HashSet<(bool, &Predicate)> {
 
 #[cfg(test)]
 mod tests {
-    use crate::idb::eval::strata::PrecedenceGraph;
-    use crate::idb::eval::{StratifiedProgram, ToGraphViz};
+    use crate::idb::eval::strata::{PrecedenceGraph, StratifiedProgram};
     use crate::parse::parse_str;
     use crate::Predicate;
     use std::str::FromStr;
