@@ -31,6 +31,14 @@ fn test_declare_infer_from_success() {
     quick_parser_check(".assert human(string). .infer mortal from human.", None);
 }
 
+#[test]
+fn test_declare_fd_success() {
+    quick_parser_check(
+        ".feature(functional_dependencies). .assert employee(string, string). .fd employee: 1 --> 2.",
+        None,
+    );
+}
+
 #[test]
 fn test_feature_success() {
     quick_parser_check(".feature(negation, comparisons).", None);
@@ -80,6 +88,46 @@ fn test_declare_assert_type_fail() {
     quick_parser_check(".assert human (name).", None);
 }
 
+// ------------------------------------------------------------------------------------------------
+// Functional dependency failure cases
+// ------------------------------------------------------------------------------------------------
+
+#[test]
+#[should_panic]
+fn test_declare_fd_no_feature() {
+    quick_parser_check(
+        ".assert employee(string, string). .fd employee: 1 --> 2.",
+        None,
+    );
+}
+
+#[test]
+#[should_panic]
+fn test_declare_fd_unknown_relation() {
+    quick_parser_check(
+        ".feature(functional_dependencies). .fd employee: 1 --> 2.",
+        None,
+    );
+}
+
+#[test]
+#[should_panic]
+fn test_declare_fd_invalid_usize_index() {
+    quick_parser_check(
+        ".feature(functional_dependencies). .assert employee(string, string). .fd employee: 1 --> 3.",
+        None,
+    );
+}
+
+#[test]
+#[should_panic]
+fn test_declare_fd_invalid_label_index() {
+    quick_parser_check(
+        ".feature(functional_dependencies). .assert employee(a: string, b: string). .fd employee: a --> c.",
+        None,
+    );
+}
+
 // ------------------------------------------------------------------------------------------------
 // General failures cases
 // ------------------------------------------------------------------------------------------------
